import os
import shutil
import tempfile
import subprocess
from subprocess import PIPE

MODE_BATCH = 0
MODE_NON_STOP = 1
MODE_SCROLL = 2
MODE_ERROR_STOP = 3
INTERACTION_MODES = ['batchmode', 'nonstopmode', 'scrollmode', 'errorstopmode']

JINJA2_ENV = {'block_start_string': '\BLOCK{',
              'block_end_string': '}',
              'variable_start_string': '\VAR{',
              'variable_end_string': '}',
              'comment_start_string': '\#{',
              'comment_end_string': '}',
              'line_statement_prefix': '%%',
              'line_comment_prefix': '%#',
              'trim_blocks': True,
              'autoescape': False}


class PDFLaTeX:
    def __init__(self, latex_src, job_name: str):
        self.latex = latex_src
        self.job_name = job_name
        self.interaction_mode = INTERACTION_MODES[MODE_BATCH]
        self.dir = None
        self.pdf_filename = None
        self.params = dict()
        self.pdf = None
        self.log = None
        
    @classmethod
    def from_texfile(cls, filename):
        prefix = os.path.basename(filename)
        prefix = os.path.splitext(prefix)[0]
        with open(filename, 'rb') as f:
            return cls.from_binarystring(f.read(), prefix)

    @classmethod
    def from_binarystring(cls, binstr: str, jobname: str):
        return cls(binstr, jobname)

    @classmethod
    def from_jinja_template(cls, jinja2_template, jobname: str = None, **render_kwargs):
        tex_src = jinja2_template.render(**render_kwargs)
        fn = jinja2_template.filename
        
        if fn is None:
            fn = jobname
            if not fn:
                raise ValueError("PDFLaTeX: if jinja template does not have attribute 'filename' set, "
                                 "'jobname' must be provided")
        return cls(tex_src, fn)

    def create_pdf(self, keep_pdf_file: bool = False, keep_log_file: bool = False, env: dict = None):
        if self.interaction_mode is not None:
            self.add_args({'-interaction-mode': self.interaction_mode})
        
        dir = self.params.get('-output-directory')
        filename = self.params.get('-jobname')
        
        if filename is None:
            filename = self.job_name
        if dir is None:
            dir = ""
        
        with tempfile.TemporaryDirectory() as td:
            self.set_output_directory(td)
            self.set_jobname('file')
    
            args = self.get_run_args()
            fp = subprocess.run(args, input=self.latex, env=env, timeout=15, stdout=PIPE, stderr=PIPE)
            with open(os.path.join(td, 'file.pdf'), 'rb') as f:
                self.pdf = f.read()
            with open(os.path.join(td, 'file.log'), 'rb') as f:
                self.log = f.read()
            if keep_log_file:
                shutil.move(os.path.join(td, 'file.log'), os.path.join(dir, filename + '.log'))
            if keep_pdf_file:
                shutil.move(os.path.join(td, 'file.pdf'), os.path.join(dir, filename + '.pdf'))
        
        return self.pdf, self.log, fp

    def get_run_args(self):
        a = [k+('='+v if v is not None else '') for k, v in self.params.items()]
        a.insert(0, 'pdflatex')
        return a
    
    def add_args(self, params: dict):
        for k in params:
            self.params[k] = params[k]
    
    def del_args(self, params):
        if isinstance(params, str):
            params = [params]

        if isinstance(params, dict) or isinstance(params, list):
            for k in params:
                if k in self.params.keys():
                    del self.params[k]
        else:
            raise ValueError('PDFLaTeX: del_cmd_params: parameter must be str, dict or list.')
    
    def set_output_directory(self, dir: str = None):
        self.generic_param_set('-output-directory', dir)

    def set_jobname(self, jobname: str = None):
        self.generic_param_set('-jobname', jobname)

    def set_output_format(self, fmt: str = None):
        if fmt and fmt not in ['pdf', 'dvi']:
            raise ValueError("PDFLaTeX: Format must be either 'pdf' or 'dvi'.")
        self.generic_param_set('-output-format', dir)
    
    def generic_param_set(self, param_name, value):
        if value is None:
            if param_name in self.params.keys():
                del self.params[param_name]
        else:
            self.params[param_name] = value
    
    def set_pdf_filename(self, pdf_filename: str = None):
        self.set_jobname(pdf_filename)
    
    def set_batchmode(self, on: bool = True):
        self.interaction_mode = INTERACTION_MODES[MODE_BATCH] if on else None

    def set_nonstopmode(self, on: bool =True):
        self.interaction_mode = INTERACTION_MODES[MODE_NON_STOP] if on else None

    def set_scrollmode(self, on: bool = True):
        self.interaction_mode = INTERACTION_MODES[MODE_SCROLL] if on else None

    def set_errorstopmode(self, on: bool = True):
        self.interaction_mode = INTERACTION_MODES[MODE_ERROR_STOP] if on else None

    def set_interaction_mode(self, mode: int = None):
        if mode is None:
            self.interaction_mode = None
        elif 0 <= mode <= 3:
            self.interaction_mode = INTERACTION_MODES[mode]
        else:
            raise ValueError('PDFLaTeX: Invalid interaction mode!')


# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['pdflatex']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=18.2,<19.0']

setup_kwargs = {
    'name': 'pdflatex',
    'version': '0.1.3',
    'description': 'Simple wrapper to calling pdflatex',
    'long_description': "# Overview\n\nThis is a simple module to execute pdflatex in an easy and clean way.\nThe pdflatex command line utility by default generates a lot of output and can create many files.\n\n## Instantiation\n\nThe PDFLaTeX class can be instantiated directly or through helpers:\n\n- **from_texfile(filename)**\n- **from_binarystring(binstr, jobname)**\n- **from_jinja2_template(jinja2_template, jobname = None, \\*\\*render_kwargs)**\n\njobname is any string that can be used to create a valid filename;\n\n\n## Examples:\n\nIn all the following examples, no files are left on the filesystem,\nunless requested with the *keep_pdf_file* and *keep_log_file* parameters\nto the *create_pdf* method.\n\n### Create PDF from .tex file\n\n    from pdflatex import PDFLaTeX\n\n    pdfl = PDFLaTeX.from_texfile('my_file.tex')\n    pdf, log, completed_process = pdfl.create_pdf(keep_pdf_file=True, keep_log_file=True)\n\nThe function **create_pdf** returns 3 results:\n1. The pdf file in a binary string;\n2. The log file generated by pdflatex as text;\n3. An instance of subprocess.CompleteProcess with the results of the pdflatex execution.\n\nAlso, **create_pdf** takes a few parameters:\n1. keep_pdf_file: an optional boolean. Default to False. If True a pdf file is maintained. Its location and name depends on the value of the -output-directory and -jobname parameters. It is also derived from the tex filename or the jinja2 template filename if no parameter is given;\n2. keep_log_file: same thing, for the log file.\n3. env: a default ENV mapping for the execution of pdflatex. You probably want to skip this.\n\n\n### Create PDF from Jinja2 Template\n\n    import os\n    import pdflatex\n    import jinja2 \n    \n    env = pdflatex.JINJA2_ENV\n    env['loader'] = jinja2.FileSystemLoader(os.path.abspath('.'))\n    env = jinja2.Environment(**env)\n    template = env.get_template('jinja.tex')\n    pdfl = pdflatex.PDFLaTeX.from_jinja2_template(template, data='Hello world!')\n    pdf, log, cp = pdfl.create_pdf()\n\nQuite self explanatory, just note that pdflatex includes a dictionary\nJINJA2_ENV with a suggestion of environment parameters for you to use with\njinja2 and LaTeX. It os defined as:\n\n    JINJA2_ENV = {'block_start_string': '\\BLOCK{',\n                  'block_end_string': '}',\n                  'variable_start_string': '\\VAR{',\n                  'variable_end_string': '}',\n                  'comment_start_string': '\\#{',\n                  'comment_end_string': '}',\n                  'line_statement_prefix': '%%',\n                  'line_comment_prefix': '%#',\n                  'trim_blocks': True,\n                  'autoescape': False }\n\n\n### Create PDF file from binary string:\n\n    import pdflatex\n\n    with open('my_file.tex', 'rb') as f:\n        pdfl = pdflatex.PDFLaTeX.from_binarystring(f.read(), 'my_file')\n    pdf, log, cp = pdfl.create_pdf()",
    'author': 'mbello',
    'author_email': 'mbello@users.noreply.github.com',
    'url': 'https://github.com/mbello/pdflatex',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
